@@ -30,78 +30,24 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
-#ifdef G_OS_WIN32
-#include <io.h>
-#endif
 
 #ifndef G_OS_WIN32
 static const char *path = "/usr/bin/gpg";
 static const char *passphrase = "no.secret";
 
-typedef struct _ExampleSession ExampleSession;
-typedef struct _ExampleSessionClass ExampleSessionClass;
-
-struct _ExampleSession {
-	GMimeSession parent_object;
-	
-};
-
-struct _ExampleSessionClass {
-	GMimeSessionClass parent_class;
-	
-};
-
-static void example_session_class_init (ExampleSessionClass *klass);
-
-static char *request_passwd (GMimeSession *session, const char *prompt,
-			     gboolean secret, const char *item,
-			     GError **err);
-
-
-static GMimeSessionClass *parent_class = NULL;
-
-
-static GType
-example_session_get_type (void)
+static gboolean
+request_passwd (GMimeCipherContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
 {
-	static GType type = 0;
-	
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (ExampleSessionClass),
-			NULL, /* base_class_init */
-			NULL, /* base_class_finalize */
-			(GClassInitFunc) example_session_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (ExampleSession),
-			0,    /* n_preallocs */
-			NULL, /* object_init */
-		};
-		
-		type = g_type_register_static (GMIME_TYPE_SESSION, "ExampleSession", &info, 0);
+	if (g_mime_stream_write_string (response, passphrase) == -1 ||
+	    g_mime_stream_write (response, "\n", 1) == -1) {
+		g_set_error (err, GMIME_ERROR, errno, "%s", g_strerror (errno));
+		return FALSE;
 	}
 	
-	return type;
-}
-
-
-static void
-example_session_class_init (ExampleSessionClass *klass)
-{
-	GMimeSessionClass *session_class = GMIME_SESSION_CLASS (klass);
-	
-	parent_class = g_type_class_ref (GMIME_TYPE_SESSION);
-	
-	session_class->request_passwd = request_passwd;
-}
-
-static char *
-request_passwd (GMimeSession *session, const char *prompt, gboolean secret, const char *item, GError **err)
-{
-	return g_strdup (passphrase);
+	return TRUE;
 }
 #endif /* ! G_OS_WIN32 */
 
@@ -337,7 +283,6 @@ remove_a_mime_part (GMimeMessage *message)
 int main (int argc, char **argv)
 {
 #ifndef G_OS_WIN32
-	GMimeSession *session;
 	GMimeCipherContext *ctx;
 #endif
 	GMimeMessage *message;
@@ -347,7 +292,7 @@ int main (int argc, char **argv)
 		printf ("Usage: a.out <message file>\n");
 		return 0;
 	} else {
-		if ((fd = open (argv[1], O_RDONLY)) == -1) {
+		if ((fd = open (argv[1], O_RDONLY, 0)) == -1) {
 			fprintf (stderr, "Cannot open message `%s': %s\n", argv[1], g_strerror (errno));
 			return 0;
 		}
@@ -363,10 +308,12 @@ int main (int argc, char **argv)
 	count_parts_in_message (message);
 	
 #ifndef G_OS_WIN32
-	/* create our cipher context (and session - which is used by the context to query the user) */
-	session = g_object_new (example_session_get_type (), NULL);
-	ctx = g_mime_gpg_context_new (session, path);
-	g_object_unref (session);
+#ifdef ENABLE_CRYPTOGRAPHY
+	/* create our cipher context */
+	ctx = g_mime_gpg_context_new (request_passwd, path);
+	
+	/* don't allow auto key-retrival */
+	g_mime_gpg_context_set_auto_key_retrieve ((GMimeGpgContext *) ctx, FALSE);
 	
 	/* set the always_trust flag so that gpg will be spawned with `gpg --always-trust` */
 	g_mime_gpg_context_set_always_trust ((GMimeGpgContext *) ctx, TRUE);
@@ -374,6 +321,7 @@ int main (int argc, char **argv)
 	/* verify any signed parts */
 	verify_signed_parts (message, ctx);
 	g_object_unref (ctx);
+#endif
 #endif
 	
 	/* add and remove parts */
