@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -44,6 +44,7 @@
 #include "gmime-gpg-context.h"
 #include "gmime-filter-charset.h"
 #include "gmime-stream-filter.h"
+#include "gmime-stream-pipe.h"
 #include "gmime-stream-mem.h"
 #include "gmime-stream-fs.h"
 #include "gmime-charset.h"
@@ -103,15 +104,6 @@ static int gpg_export_keys (GMimeCipherContext *ctx, GPtrArray *keys,
 static GMimeCipherContextClass *parent_class = NULL;
 
 
-#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 18
-static void
-g_set_error_literal (GError **err, GQuark domain, gint code, const gchar *message)
-{
-	g_set_error (err, domain, code, "%s", message);
-}
-#endif
-
-
 GType
 g_mime_gpg_context_get_type (void)
 {
@@ -162,8 +154,9 @@ g_mime_gpg_context_init (GMimeGpgContext *ctx, GMimeGpgContextClass *klass)
 {
 	GMimeCipherContext *cipher = (GMimeCipherContext *) ctx;
 	
-	ctx->path = NULL;
+	ctx->auto_key_retrieve = FALSE;
 	ctx->always_trust = FALSE;
+	ctx->path = NULL;
 	
 	cipher->sign_protocol = "application/pgp-signature";
 	cipher->encrypt_protocol = "application/pgp-encrypted";
@@ -255,12 +248,12 @@ enum _GpgCtxMode {
 
 struct _GpgCtx {
 	enum _GpgCtxMode mode;
-	GMimeSession *session;
 	GHashTable *userid_hint;
+	GMimeGpgContext *ctx;
 	pid_t pid;
 	
-	char *path;
 	char *userid;
+	char *sigfile;
 	GPtrArray *recipients;
 	GMimeCipherHash hash;
 	
@@ -268,7 +261,7 @@ struct _GpgCtx {
 	int stdout_fd;
 	int stderr_fd;
 	int status_fd;
-	int secret_fd;  /* used for sign/decrypt/verify */
+	int passwd_fd;  /* only needed for sign/decrypt */
 	
 	/* status-fd buffer */
 	char *statusbuf;
@@ -276,9 +269,7 @@ struct _GpgCtx {
 	guint statusleft;
 	
 	char *need_id;
-	char *passwd;
 	
-	GMimeStream *sigstream;
 	GMimeStream *istream;
 	GMimeStream *ostream;
 	
@@ -299,7 +290,6 @@ struct _GpgCtx {
 	unsigned int always_trust:1;
 	unsigned int armor:1;
 	unsigned int need_passwd:1;
-	unsigned int send_passwd:1;
 	
 	unsigned int bad_passwds:2;
 	
@@ -310,11 +300,11 @@ struct _GpgCtx {
 	unsigned int nopubkey:1;
 	unsigned int nodata:1;
 	
-	unsigned int padding:14;
+	unsigned int padding:15;
 };
 
 static struct _GpgCtx *
-gpg_ctx_new (GMimeSession *session, const char *path)
+gpg_ctx_new (GMimeGpgContext *ctx)
 {
 	struct _GpgCtx *gpg;
 	const char *charset;
@@ -322,8 +312,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	
 	gpg = g_slice_new (struct _GpgCtx);
 	gpg->mode = GPG_CTX_MODE_SIGN;
-	gpg->session = session;
-	g_object_ref (session);
+	gpg->ctx = ctx;
 	gpg->userid_hint = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	gpg->complete = FALSE;
 	gpg->seen_eof1 = TRUE;
@@ -333,8 +322,8 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->flushed = FALSE;
 	gpg->exited = FALSE;
 	
-	gpg->path = g_strdup (path);
 	gpg->userid = NULL;
+	gpg->sigfile = NULL;
 	gpg->recipients = NULL;
 	gpg->hash = GMIME_CIPHER_HASH_DEFAULT;
 	gpg->always_trust = FALSE;
@@ -344,7 +333,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->stdout_fd = -1;
 	gpg->stderr_fd = -1;
 	gpg->status_fd = -1;
-	gpg->secret_fd = -1;
+	gpg->passwd_fd = -1;
 	
 	gpg->statusbuf = g_malloc (128);
 	gpg->statusptr = gpg->statusbuf;
@@ -352,9 +341,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	
 	gpg->bad_passwds = 0;
 	gpg->need_passwd = FALSE;
-	gpg->send_passwd = FALSE;
 	gpg->need_id = NULL;
-	gpg->passwd = NULL;
 	
 	gpg->nodata = FALSE;
 	gpg->badsig = FALSE;
@@ -366,7 +353,6 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->signers = NULL;
 	gpg->signer = (GMimeSigner *) &gpg->signers;
 	
-	gpg->sigstream = NULL;
 	gpg->istream = NULL;
 	gpg->ostream = NULL;
 	
@@ -436,6 +422,13 @@ gpg_ctx_add_recipient (struct _GpgCtx *gpg, const char *keyid)
 	g_ptr_array_add (gpg->recipients, g_strdup (keyid));
 }
 
+static void
+gpg_ctx_set_sigfile (struct _GpgCtx *gpg, const char *sigfile)
+{
+	g_free (gpg->sigfile);
+	gpg->sigfile = g_strdup (sigfile);
+}
+
 static void
 gpg_ctx_set_armor (struct _GpgCtx *gpg, gboolean armor)
 {
@@ -461,15 +454,6 @@ gpg_ctx_set_ostream (struct _GpgCtx *gpg, GMimeStream *ostream)
 	gpg->seen_eof1 = FALSE;
 }
 
-static void
-gpg_ctx_set_sigstream (struct _GpgCtx *gpg, GMimeStream *sigstream)
-{
-	g_object_ref (sigstream);
-	if (gpg->sigstream)
-		g_object_unref (gpg->sigstream);
-	gpg->sigstream = sigstream;
-}
-
 static const char *
 gpg_ctx_get_diagnostics (struct _GpgCtx *gpg)
 {
@@ -488,15 +472,12 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 	GMimeSigner *signer, *next;
 	guint i;
 	
-	if (gpg->session)
-		g_object_unref (gpg->session);
-	
 	g_hash_table_destroy (gpg->userid_hint);
 	
-	g_free (gpg->path);
-	
 	g_free (gpg->userid);
 	
+	g_free (gpg->sigfile);
+	
 	if (gpg->recipients) {
 		for (i = 0; i < gpg->recipients->len; i++)
 			g_free (gpg->recipients->pdata[i]);
@@ -512,21 +493,13 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 		close (gpg->stderr_fd);
 	if (gpg->status_fd != -1)
 		close (gpg->status_fd);
-	if (gpg->secret_fd != -1)
-		close (gpg->secret_fd);
+	if (gpg->passwd_fd != -1)
+		close (gpg->passwd_fd);
 	
 	g_free (gpg->statusbuf);
 	
 	g_free (gpg->need_id);
 	
-	if (gpg->passwd) {
-		memset (gpg->passwd, 0, strlen (gpg->passwd));
-		g_free (gpg->passwd);
-	}
-	
-	if (gpg->sigstream)
-		g_object_unref (gpg->sigstream);
-	
 	if (gpg->istream)
 		g_object_unref (gpg->istream);
 	
@@ -572,134 +545,118 @@ gpg_hash_str (GMimeCipherHash hash)
 	}
 }
 
-static char **
-gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, int secret_fd, char ***strv)
+static GPtrArray *
+gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, char **sfd, int passwd_fd, char **pfd)
 {
 	const char *hash_str;
-	char **argv, *buf;
-	GPtrArray *args;
-	int v = 0;
+	GPtrArray *argv;
+	char *buf;
 	guint i;
 	
-	*strv = g_new (char *, 3);
+	argv = g_ptr_array_new ();
+	g_ptr_array_add (argv, "gpg");
 	
-	args = g_ptr_array_new ();
-	g_ptr_array_add (args, "gpg");
-	
-	g_ptr_array_add (args, "--verbose");
-	g_ptr_array_add (args, "--no-secmem-warning");
-	g_ptr_array_add (args, "--no-greeting");
-	g_ptr_array_add (args, "--no-tty");
-	
-	if (!gpg->need_passwd) {
+	g_ptr_array_add (argv, "--verbose");
+	g_ptr_array_add (argv, "--no-secmem-warning");
+	g_ptr_array_add (argv, "--no-greeting");
+	g_ptr_array_add (argv, "--no-tty");
+	if (passwd_fd == -1) {
 		/* only use batch mode if we don't intend on using the
-                   interactive --command-fd option to send it the
-                   user's password */
-		g_ptr_array_add (args, "--batch");
-		g_ptr_array_add (args, "--yes");
+                   interactive --command-fd option */
+		g_ptr_array_add (argv, "--batch");
+		g_ptr_array_add (argv, "--yes");
 	}
 	
-	g_ptr_array_add (args, "--charset=UTF-8");
+	g_ptr_array_add (argv, "--charset=UTF-8");
 	
-	(*strv)[v++] = buf = g_strdup_printf ("--status-fd=%d", status_fd);
-	g_ptr_array_add (args, buf);
+	*sfd = buf = g_strdup_printf ("--status-fd=%d", status_fd);
+	g_ptr_array_add (argv, buf);
 	
-	if (gpg->need_passwd) {
-		(*strv)[v++] = buf = g_strdup_printf ("--command-fd=%d", secret_fd);
-		g_ptr_array_add (args, buf);
+	if (passwd_fd != -1) {
+		*pfd = buf = g_strdup_printf ("--command-fd=%d", passwd_fd);
+		g_ptr_array_add (argv, buf);
 	}
 	
 	switch (gpg->mode) {
 	case GPG_CTX_MODE_SIGN:
-		g_ptr_array_add (args, "--sign");
-		g_ptr_array_add (args, "--detach");
+		g_ptr_array_add (argv, "--sign");
+		g_ptr_array_add (argv, "--detach");
 		if (gpg->armor)
-			g_ptr_array_add (args, "--armor");
+			g_ptr_array_add (argv, "--armor");
 		hash_str = gpg_hash_str (gpg->hash);
 		if (hash_str)
-			g_ptr_array_add (args, (char *) hash_str);
+			g_ptr_array_add (argv, (char *) hash_str);
 		if (gpg->userid) {
-			g_ptr_array_add (args, "-u");
-			g_ptr_array_add (args, (char *) gpg->userid);
+			g_ptr_array_add (argv, "-u");
+			g_ptr_array_add (argv, (char *) gpg->userid);
 		}
-		g_ptr_array_add (args, "--output");
-		g_ptr_array_add (args, "-");
+		g_ptr_array_add (argv, "--output");
+		g_ptr_array_add (argv, "-");
 		break;
 	case GPG_CTX_MODE_VERIFY:
-		if (!g_mime_session_is_online (gpg->session)) {
-			/* this is a deprecated flag to gpg since 1.0.7 */
-			/*g_ptr_array_add (args, "--no-auto-key-retrieve");*/
-			g_ptr_array_add (args, "--keyserver-options");
-			g_ptr_array_add (args, "no-auto-key-retrieve");
+		if (!gpg->ctx->auto_key_retrieve) {
+			/* don't allow gpg to fetch keys from a server... */
+			g_ptr_array_add (argv, "--keyserver-options");
+			g_ptr_array_add (argv, "no-auto-key-retrieve");
 		}
 		
-		g_ptr_array_add (args, "--enable-special-filenames");
-		g_ptr_array_add (args, "--verify");
-		g_ptr_array_add (args, "--");
-		
-		/* signature stream must come first */
-		(*strv)[v++] = buf = g_strdup_printf ("-&%d", secret_fd);
-		g_ptr_array_add (args, buf);
-		
-		/* followed by the content stream (in this case, stdin) */
-		g_ptr_array_add (args, "-");
+		g_ptr_array_add (argv, "--verify");
+		if (gpg->sigfile)
+			g_ptr_array_add (argv, gpg->sigfile);
+		g_ptr_array_add (argv, "-");
 		break;
 	case GPG_CTX_MODE_SIGN_ENCRYPT:
-		g_ptr_array_add (args, "--sign");
+		g_ptr_array_add (argv,  "--sign");
 		
 		/* fall thru... */
 	case GPG_CTX_MODE_ENCRYPT:
-		g_ptr_array_add (args, "--encrypt");
+		g_ptr_array_add (argv,  "--encrypt");
 		
 		if (gpg->armor)
-			g_ptr_array_add (args, "--armor");
+			g_ptr_array_add (argv, "--armor");
 		
 		if (gpg->always_trust)
-			g_ptr_array_add (args, "--always-trust");
+			g_ptr_array_add (argv, "--always-trust");
 		
 		if (gpg->userid) {
-			g_ptr_array_add (args, "-u");
-			g_ptr_array_add (args, (char *) gpg->userid);
+			g_ptr_array_add (argv, "-u");
+			g_ptr_array_add (argv, (char *) gpg->userid);
 		}
 		
 		if (gpg->recipients) {
 			for (i = 0; i < gpg->recipients->len; i++) {
-				g_ptr_array_add (args, "-r");
-				g_ptr_array_add (args, gpg->recipients->pdata[i]);
+				g_ptr_array_add (argv, "-r");
+				g_ptr_array_add (argv, gpg->recipients->pdata[i]);
 			}
 		}
-		g_ptr_array_add (args, "--output");
-		g_ptr_array_add (args, "-");
+		g_ptr_array_add (argv, "--output");
+		g_ptr_array_add (argv, "-");
 		break;
 	case GPG_CTX_MODE_DECRYPT:
-		g_ptr_array_add (args, "--decrypt");
-		g_ptr_array_add (args, "--output");
-		g_ptr_array_add (args, "-");
+		g_ptr_array_add (argv, "--decrypt");
+		g_ptr_array_add (argv, "--output");
+		g_ptr_array_add (argv, "-");
 		break;
 	case GPG_CTX_MODE_IMPORT:
-		g_ptr_array_add (args, "--import");
-		g_ptr_array_add (args, "-");
+		g_ptr_array_add (argv, "--import");
+		g_ptr_array_add (argv, "-");
 		break;
 	case GPG_CTX_MODE_EXPORT:
 		if (gpg->armor)
-			g_ptr_array_add (args, "--armor");
-		g_ptr_array_add (args, "--export");
+			g_ptr_array_add (argv, "--armor");
+		g_ptr_array_add (argv, "--export");
 		for (i = 0; i < gpg->recipients->len; i++)
-			g_ptr_array_add (args, gpg->recipients->pdata[i]);
+			g_ptr_array_add (argv, gpg->recipients->pdata[i]);
 		break;
 	}
 	
 #if d(!)0
-	for (i = 0; i < args->len; i++)
-		printf ("%s ", (char *) args->pdata[i]);
+	for (i = 0; i < argv->len; i++)
+		printf ("%s ", (char *) argv->pdata[i]);
 	printf ("\n");
 #endif
 	
-	g_ptr_array_add (args, NULL);
-	(*strv)[v] = NULL;
-	
-	argv = (char **) args->pdata;
-	g_ptr_array_free (args, FALSE);
+	g_ptr_array_add (argv, NULL);
 	
 	return argv;
 }
@@ -707,27 +664,28 @@ gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, int secret_fd, char ***str
 static int
 gpg_ctx_op_start (struct _GpgCtx *gpg)
 {
+	char *status_fd = NULL, *passwd_fd = NULL;
 	int i, maxfd, errnosave, fds[10];
-	char **argv, **strv = NULL;
+	GPtrArray *argv;
 	int flags;
 	
 	for (i = 0; i < 10; i++)
 		fds[i] = -1;
 	
-	maxfd = (gpg->need_passwd || gpg->sigstream) ? 10 : 8;
+	maxfd = gpg->need_passwd ? 10 : 8;
 	for (i = 0; i < maxfd; i += 2) {
 		if (pipe (fds + i) == -1)
 			goto exception;
 	}
 	
-	argv = gpg_ctx_get_argv (gpg, fds[7], fds[8], &strv);
+	argv = gpg_ctx_get_argv (gpg, fds[7], &status_fd, fds[8], &passwd_fd);
 	
 	if (!(gpg->pid = fork ())) {
 		/* child process */
 		
-		if ((dup2 (fds[0], STDIN_FILENO) < 0) ||
-		    (dup2 (fds[3], STDOUT_FILENO) < 0) ||
-		    (dup2 (fds[5], STDERR_FILENO) < 0)) {
+		if ((dup2 (fds[0], STDIN_FILENO) < 0 ) ||
+		    (dup2 (fds[3], STDOUT_FILENO) < 0 ) ||
+		    (dup2 (fds[5], STDERR_FILENO) < 0 )) {
 			_exit (255);
 		}
 		
@@ -744,19 +702,20 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 		}
 		
 		/* run gpg */
-		execvp (gpg->path, argv);
+		execvp (gpg->ctx->path, (char **) argv->pdata);
 		_exit (255);
 	} else if (gpg->pid < 0) {
-		g_strfreev (strv);
-		g_free (argv);
+		g_ptr_array_free (argv, TRUE);
+		g_free (status_fd);
+		g_free (passwd_fd);
 		goto exception;
 	}
 	
-	/* parent process */
-	
-	g_strfreev (strv);
-	g_free (argv);
+	g_ptr_array_free (argv, TRUE);
+	g_free (status_fd);
+	g_free (passwd_fd);
 	
+	/* Parent */
 	close (fds[0]);
 	gpg->stdin_fd = fds[1];
 	gpg->stdout_fd = fds[2];
@@ -765,12 +724,11 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	close (fds[5]);
 	gpg->status_fd = fds[6];
 	close (fds[7]);
-	
-	if (fds[8] != -1) {
-		flags = (flags = fcntl (fds[9], F_GETFL)) == -1 ? O_WRONLY : flags;
-		fcntl (fds[9], F_SETFL, flags | O_NONBLOCK);
-		gpg->secret_fd = fds[9];
+	if (gpg->need_passwd) {
 		close (fds[8]);
+		gpg->passwd_fd = fds[9];
+		flags = (flags = fcntl (gpg->passwd_fd, F_GETFL)) == -1 ? O_WRONLY : flags;
+		fcntl (gpg->passwd_fd, F_SETFL, flags | O_NONBLOCK);
 	}
 	
 	flags = (flags = fcntl (gpg->stdin_fd, F_GETFL)) == -1 ? O_WRONLY : flags;
@@ -791,7 +749,7 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	
 	errnosave = errno;
 	
-	for (i = 0; i < maxfd; i++) {
+	for (i = 0; i < 10; i++) {
 		if (fds[i] != -1)
 			close (fds[i]);
 	}
@@ -825,34 +783,6 @@ next_token (char *in, char **token)
 	return inptr;
 }
 
-/**
- * gpg_ctx_add_signer:
- * @gpg: GnuPG context
- * @info: a string with the signer's info
- *
- * Parses GOODSIG, BADSIG, EXPSIG, EXPKEYSIG, and REVKEYSIG status messages
- * into a newly allocated #GMimeSigner, adding it to @gpg's signer list.
- *
- * Returns: the newly added signer for the caller to add more info to.
- **/
-static GMimeSigner *
-gpg_ctx_add_signer (struct _GpgCtx *gpg, char *info)
-{
-	GMimeSigner *signer;
-	
-	signer = g_mime_signer_new ();
-	gpg->signer->next = signer;
-	gpg->signer = signer;
-	
-	/* get the key id of the signer */
-	info = next_token (info, &signer->keyid);
-	
-	/* the rest of the string is the signer's name */
-	signer->name = g_strdup (info);
-	
-	return signer;
-}
-
 static void
 gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 {
@@ -861,20 +791,33 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 	if (!strncmp (status, "SIG_ID ", 7)) {
 		/* not sure if this contains anything we care about... */
 	} else if (!strncmp (status, "GOODSIG ", 8)) {
-		signer = gpg_ctx_add_signer (gpg, status + 8);
 		gpg->goodsig = TRUE;
+		status += 8;
+		
+		signer = g_mime_signer_new ();
+		signer->status = GMIME_SIGNER_STATUS_GOOD;
+		gpg->signer->next = signer;
+		gpg->signer = signer;
+		
+		/* get the key id of the signer */
+		status = next_token (status, &signer->keyid);
+		
+		/* the rest of the string is the signer's name */
+		signer->name = g_strdup (status);
 	} else if (!strncmp (status, "BADSIG ", 7)) {
-		signer = gpg_ctx_add_signer (gpg, status + 7);
 		gpg->badsig = TRUE;
-	} else if (!strncmp (status, "EXPSIG ", 7)) {
-		signer = gpg_ctx_add_signer (gpg, status + 7);
-		signer->errors |= GMIME_SIGNER_ERROR_EXPSIG;
-	} else if (!strncmp (status, "EXPKEYSIG ", 10)) {
-		signer = gpg_ctx_add_signer (gpg, status + 10);
-		signer->errors |= GMIME_SIGNER_ERROR_EXPKEYSIG;
-	} else if (!strncmp (status, "REVKEYSIG ", 10)) {
-		signer = gpg_ctx_add_signer (gpg, status + 10);
-		signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
+		status += 7;
+		
+		signer = g_mime_signer_new ();
+		signer->status = GMIME_SIGNER_STATUS_BAD;
+		gpg->signer->next = signer;
+		gpg->signer = signer;
+		
+		/* get the key id of the signer */
+		status = next_token (status, &signer->keyid);
+		
+		/* the rest of the string is the signer's name */
+		signer->name = g_strdup (status);
 	} else if (!strncmp (status, "ERRSIG ", 7)) {
 		/* Note: NO_PUBKEY often comes after an ERRSIG */
 		gpg->errsig = TRUE;
@@ -898,7 +841,7 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 		status = next_token (status, NULL);
 		
 		/* get the signature expiration date (or 0 for never) */
-		signer->expires = strtoul (status, NULL, 10);
+		signer->sig_expires = strtoul (status, NULL, 10);
 		status = next_token (status, NULL);
 		
 		/* the last token is the 'rc' which we don't care about */
@@ -906,6 +849,13 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 		/* the only token is the keyid, but we've already got it */
 		gpg->signer->errors |= GMIME_SIGNER_ERROR_NO_PUBKEY;
 		gpg->nopubkey = TRUE;
+	} else if (!strncmp (status, "EXPSIG", 6)) {
+		/* FIXME: see what else we can glean from this... */
+		gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPSIG;
+	} else if (!strncmp (status, "EXPKEYSIG", 9)) {
+		gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPKEYSIG;
+	} else if (!strncmp (status, "REVKEYSIG", 9)) {
+		gpg->signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
 	} else if (!strncmp (status, "VALIDSIG ", 9)) {
 		char *inend;
 		
@@ -921,14 +871,14 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 		status = next_token (status, NULL);
 		
 		/* the third token is the signature creation date (or 0 for unknown?) */
-		signer->created = strtoul (status, &inend, 10);
+		signer->sig_created = strtoul (status, &inend, 10);
 		if (inend == status || *inend != ' ')
 			return;
 		
 		status = inend + 1;
 		
 		/* the fourth token is the signature expiration date (or 0 for never) */
-		signer->expires = strtoul (status, NULL, 10);
+		signer->sig_expires = strtoul (status, NULL, 10);
 		
 		/* ignore the rest... */
 	} else if (!strncmp (status, "TRUST_", 6)) {
@@ -1047,12 +997,23 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		g_free (gpg->need_id);
 		gpg->need_id = userid;
 	} else if (!strncmp (status, "GET_HIDDEN ", 11)) {
+		GMimeStream *filtered_stream, *passwd;
+		GMimeCipherContext *ctx;
+		GMimeFilter *filter;
+		const char *charset;
 		char *prompt = NULL;
-		char *passwd = NULL;
 		const char *name;
+		gboolean ok;
 		
 		status += 11;
 		
+		ctx = (GMimeCipherContext *) gpg->ctx;
+		if (!ctx->request_passwd) {
+			/* can't ask for a passwd w/o a way to request it from the user... */
+			g_set_error_literal (err, GMIME_ERROR, ECANCELED, _("Canceled."));
+			return -1;
+		}
+		
 		if (!(name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id)))
 			name = gpg->userid;
 		
@@ -1070,39 +1031,41 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			return -1;
 		}
 		
-		if ((passwd = g_mime_session_request_passwd (gpg->session, prompt, TRUE, gpg->need_id, err))) {
-			g_free (prompt);
+		/* create a stream for the application to write the passwd to */
+		passwd = g_mime_stream_pipe_new (gpg->passwd_fd);
+		g_mime_stream_pipe_set_owner ((GMimeStreamPipe *) passwd, FALSE);
+		
+		if (!gpg->utf8) {
+			/* we'll need to transcode the UTF-8 password that the application
+			 * will write to our stream into the locale charset used by gpg */
+			filtered_stream = g_mime_stream_filter_new (passwd);
+			g_object_unref (passwd);
 			
-			if (!gpg->utf8) {
-				char *locale_passwd;
-				
-				if ((locale_passwd = g_locale_from_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
-					memset (passwd, 0, strlen (passwd));
-					g_free (passwd);
-					passwd = locale_passwd;
-				}
-			}
+			charset = g_mime_locale_charset ();
+			filter = g_mime_filter_charset_new ("UTF-8", charset);
 			
-			gpg->passwd = g_strdup_printf ("%s\n", passwd);
-			memset (passwd, 0, strlen (passwd));
-			g_free (passwd);
+			g_mime_stream_filter_add ((GMimeStreamFilter *) filtered_stream, filter);
+			g_object_unref (filter);
 			
-			gpg->send_passwd = TRUE;
-		} else {
-			if (err && *err == NULL)
-				g_set_error_literal (err, GMIME_ERROR, ECANCELED, _("Canceled."));
-			
-			g_free (prompt);
-			
-			return -1;
+			passwd = filtered_stream;
 		}
+		
+		if ((ok = ctx->request_passwd (ctx, name, prompt, gpg->bad_passwds > 0, passwd, err))) {
+			if (g_mime_stream_flush (passwd) == -1) {
+				
+				ok = FALSE;
+			}
+		}
+		g_object_unref (passwd);
+		g_free (prompt);
+		
+		if (!ok)
+			return -1;
 	} else if (!strncmp (status, "GOOD_PASSPHRASE", 15)) {
 		gpg->bad_passwds = 0;
 	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
 		gpg->bad_passwds++;
 		
-		g_mime_session_forget_passwd (gpg->session, gpg->userid, NULL);
-		
 		if (gpg->bad_passwds == 3) {
 			g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_BAD_PASSWORD,
 					     _("Failed to unlock secret key: 3 bad passphrases given."));
@@ -1248,7 +1211,7 @@ nearest_pow (size_t num)
 static inline size_t
 next_alloc_size (size_t n)
 {
-	return ((n + 63) / 64) * 64;
+	return (n + 63) & ~63;
 }
 #endif
 
@@ -1274,7 +1237,6 @@ enum {
 	GPG_STDOUT_FD,
 	GPG_STDERR_FD,
 	GPG_STATUS_FD,
-	GPG_SECRET_FD,
 	GPG_N_FDS
 };
 
@@ -1386,9 +1348,6 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	pfds[GPG_STDIN_FD].fd = gpg->stdin_fd;
 	pfds[GPG_STDIN_FD].events = POLLOUT;
 	
-	pfds[GPG_SECRET_FD].fd = gpg->secret_fd;
-	pfds[GPG_SECRET_FD].events = POLLOUT;
-	
 	do {
 		for (n = 0; n < GPG_N_FDS; n++)
 			pfds[n].revents = 0;
@@ -1441,7 +1400,6 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		do {
 			nread = read (gpg->stdout_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
-		
 		if (nread == -1)
 			goto exception;
 		
@@ -1462,7 +1420,6 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		do {
 			nread = read (gpg->stderr_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
-		
 		if (nread == -1)
 			goto exception;
 		
@@ -1473,62 +1430,6 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if ((pfds[GPG_SECRET_FD].revents & (POLLOUT | POLLHUP))) {
-		if (gpg->need_passwd && gpg->send_passwd) {
-			size_t n, nwritten = 0;
-			ssize_t w;
-			
-			d(printf ("sending gpg our passphrase...\n"));
-			
-			/* send the passphrase to gpg */
-			n = strlen (gpg->passwd);
-			do {
-				do {
-					w = write (gpg->secret_fd, gpg->passwd + nwritten, n - nwritten);
-				} while (w == -1 && (errno == EINTR || errno == EAGAIN));
-				
-				if (w > 0)
-					nwritten += w;
-			} while (nwritten < n && w != -1);
-			
-			/* zero and free our passwd buffer */
-			memset (gpg->passwd, 0, n);
-			g_free (gpg->passwd);
-			gpg->passwd = NULL;
-			
-			if (w == -1)
-				goto exception;
-			
-			gpg->send_passwd = FALSE;
-		} else if (gpg->mode == GPG_CTX_MODE_VERIFY) {
-			ssize_t nread, w, nwritten = 0;
-			char buffer[4096];
-			
-			d(printf ("sending gpg our digital signature...\n"));
-			
-			/* write our stream to gpg's sig fd */
-			nread = g_mime_stream_read (gpg->sigstream, buffer, sizeof (buffer));
-			if (nread > 0) {
-				do {
-					do {
-						w = write (gpg->secret_fd, buffer + nwritten, nread - nwritten);
-					} while (w == -1 && (errno == EINTR || errno == EAGAIN));
-					
-					if (w > 0)
-						nwritten += w;
-				} while (nwritten < nread && w != -1);
-				
-				if (w == -1)
-					goto exception;
-			}
-			
-			if (g_mime_stream_eos (gpg->sigstream)) {
-				close (gpg->secret_fd);
-				gpg->secret_fd = -1;
-			}
-		}
-	}
-	
 	if ((pfds[GPG_STDIN_FD].revents & (POLLOUT | POLLHUP)) && gpg->istream) {
 		char buffer[4096];
 		ssize_t nread;
@@ -1699,7 +1600,7 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_SIGN);
 	gpg_ctx_set_hash (gpg, hash);
 	gpg_ctx_set_armor (gpg, TRUE);
@@ -1748,6 +1649,35 @@ gpg_sign (GMimeCipherContext *context, const char *userid, GMimeCipherHash hash,
 }
 
 
+static char *
+swrite (GMimeStream *istream)
+{
+	GMimeStream *ostream;
+	char *template;
+	int fd, ret;
+	
+	template = g_build_filename (g_get_tmp_dir (), "gmime-pgp.XXXXXX", NULL);
+	if ((fd = mkstemp (template)) == -1) {
+		g_free (template);
+		return NULL;
+	}
+	
+	ostream = g_mime_stream_fs_new (fd);
+	if ((ret = g_mime_stream_write_to_stream (istream, ostream)) != -1) {
+		if ((ret = g_mime_stream_flush (ostream)) != -1)
+			ret = g_mime_stream_close (ostream);
+	}
+	g_object_unref (ostream);
+	
+	if (ret == -1) {
+		unlink (template);
+		g_free (template);
+		return NULL;
+	}
+	
+	return template;
+}
+
 static GMimeSignatureValidity *
 gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	    GMimeStream *istream, GMimeStream *sigstream,
@@ -1757,28 +1687,38 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	GMimeSignatureValidity *validity;
 	const char *diagnostics;
 	struct _GpgCtx *gpg;
+	char *sigfile = NULL;
 	gboolean valid;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	if (sigstream != NULL) {
+		/* We are going to verify a detached signature so save
+		   the signature to a temp file. */
+		if (!(sigfile = swrite (sigstream))) {
+			g_set_error (err, GMIME_ERROR, errno,
+				     _("Cannot verify message signature: "
+				       "could not create temp file: %s"),
+				     g_strerror (errno));
+			return NULL;
+		}
+	}
+	
+	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_VERIFY);
-	gpg_ctx_set_sigstream (gpg, sigstream);
-	gpg_ctx_set_istream (gpg, istream);
 	gpg_ctx_set_hash (gpg, hash);
+	gpg_ctx_set_sigfile (gpg, sigfile);
+	gpg_ctx_set_istream (gpg, istream);
 	
 	if (gpg_ctx_op_start (gpg) == -1) {
 		g_set_error (err, GMIME_ERROR, errno,
 			     _("Failed to execute gpg: %s"),
 			     errno ? g_strerror (errno) : _("Unknown"));
-		gpg_ctx_free (gpg);
-		
-		return NULL;
+		goto exception;
 	}
 	
 	while (!gpg_ctx_op_complete (gpg)) {
 		if (gpg_ctx_op_step (gpg, err) == -1) {
 			gpg_ctx_op_cancel (gpg);
-			gpg_ctx_free (gpg);
-			return NULL;
+			goto exception;
 		}
 	}
 	
@@ -1805,7 +1745,23 @@ gpg_verify (GMimeCipherContext *context, GMimeCipherHash hash,
 	
 	gpg_ctx_free (gpg);
 	
+	if (sigfile) {
+		unlink (sigfile);
+		g_free (sigfile);
+	}
+	
 	return validity;
+	
+ exception:
+	
+	gpg_ctx_free (gpg);
+	
+	if (sigfile) {
+		unlink (sigfile);
+		g_free (sigfile);
+	}
+	
+	return NULL;
 }
 
 
@@ -1818,7 +1774,7 @@ gpg_encrypt (GMimeCipherContext *context, gboolean sign, const char *userid,
 	struct _GpgCtx *gpg;
 	guint i;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg = gpg_ctx_new (ctx);
 	if (sign)
 		gpg_ctx_set_mode (gpg, GPG_CTX_MODE_SIGN_ENCRYPT);
 	else
@@ -1880,7 +1836,7 @@ gpg_decrypt (GMimeCipherContext *context, GMimeStream *istream,
 	struct _GpgCtx *gpg;
 	int save;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_DECRYPT);
 	gpg_ctx_set_istream (gpg, istream);
 	gpg_ctx_set_ostream (gpg, ostream);
@@ -1947,7 +1903,7 @@ gpg_import_keys (GMimeCipherContext *context, GMimeStream *istream, GError **err
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_IMPORT);
 	gpg_ctx_set_istream (gpg, istream);
 	
@@ -1995,7 +1951,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 	struct _GpgCtx *gpg;
 	guint i;
 	
-	gpg = gpg_ctx_new (context->session, ctx->path);
+	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_EXPORT);
 	gpg_ctx_set_armor (gpg, TRUE);
 	gpg_ctx_set_ostream (gpg, ostream);
@@ -2044,7 +2000,7 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
 
 /**
  * g_mime_gpg_context_new:
- * @session: a #GMimeSession
+ * @request_passwd: a #GMimePasswordRequestFunc
  * @path: path to gpg binary
  *
  * Creates a new gpg cipher context object.
@@ -2052,25 +2008,57 @@ gpg_export_keys (GMimeCipherContext *context, GPtrArray *keys, GMimeStream *ostr
  * Returns: a new gpg cipher context object.
  **/
 GMimeCipherContext *
-g_mime_gpg_context_new (GMimeSession *session, const char *path)
+g_mime_gpg_context_new (GMimePasswordRequestFunc request_passwd, const char *path)
 {
 	GMimeCipherContext *cipher;
 	GMimeGpgContext *ctx;
 	
-	g_return_val_if_fail (GMIME_IS_SESSION (session), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 	
 	ctx = g_object_newv (GMIME_TYPE_GPG_CONTEXT, 0, NULL);
 	ctx->path = g_strdup (path);
 	
 	cipher = (GMimeCipherContext *) ctx;
-	cipher->session = session;
-	g_object_ref (session);
+	cipher->request_passwd = request_passwd;
 	
 	return cipher;
 }
 
 
+/**
+ * g_mime_gpg_context_get_auto_key_retrieve:
+ * @ctx: a #GMimeGpgContext
+ *
+ * Gets the @auto_key_retrieve flag on the gpg context.
+ *
+ * Returns: the @auto_key_retrieve flag on the gpg context.
+ **/
+gboolean
+g_mime_gpg_context_get_auto_key_retrieve (GMimeGpgContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_GPG_CONTEXT (ctx), FALSE);
+	
+	return ctx->auto_key_retrieve;
+}
+
+
+/**
+ * g_mime_gpg_context_set_auto_key_retrieve:
+ * @ctx: a #GMimeGpgContext
+ * @auto_key_retrieve: auto-retrieve keys from a keys server
+ *
+ * Sets the @auto_key_retrieve flag on the gpg context which is used
+ * for signature verification.
+ **/
+void
+g_mime_gpg_context_set_auto_key_retrieve (GMimeGpgContext *ctx, gboolean auto_key_retrieve)
+{
+	g_return_if_fail (GMIME_IS_GPG_CONTEXT (ctx));
+	
+	ctx->auto_key_retrieve = auto_key_retrieve;
+}
+
+
 /**
  * g_mime_gpg_context_get_always_trust:
  * @ctx: a #GMimeGpgContext
