@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -30,6 +30,17 @@ G_BEGIN_DECLS
 typedef struct _GMimeHeader GMimeHeader;
 
 
+/**
+ * GMimeHeaderForeachFunc:
+ * @name: The field name.
+ * @value: The field value.
+ * @user_data: The user-supplied callback data.
+ *
+ * Function signature for the callback to g_mime_header_list_foreach().
+ **/
+typedef void (* GMimeHeaderForeachFunc) (const char *name, const char *value, gpointer user_data);
+
+
 /**
  * GMimeHeaderList:
  *
@@ -72,10 +83,11 @@ gboolean g_mime_header_iter_last (GMimeHeaderIter *iter);
 gboolean g_mime_header_iter_next (GMimeHeaderIter *iter);
 gboolean g_mime_header_iter_prev (GMimeHeaderIter *iter);
 
-gint64 g_mime_header_iter_get_offset (GMimeHeaderIter *iter);
+/*gint64 g_mime_header_iter_get_offset (GMimeHeaderIter *iter);*/
 const char *g_mime_header_iter_get_name (GMimeHeaderIter *iter);
 gboolean g_mime_header_iter_set_value (GMimeHeaderIter *iter, const char *value);
 const char *g_mime_header_iter_get_value (GMimeHeaderIter *iter);
+
 gboolean g_mime_header_iter_remove (GMimeHeaderIter *iter);
 
 
@@ -97,6 +109,9 @@ GMimeHeaderList *g_mime_header_list_new (void);
 
 void g_mime_header_list_destroy (GMimeHeaderList *headers);
 
+void g_mime_header_list_set_stream (GMimeHeaderList *headers, GMimeStream *stream);
+GMimeStream *g_mime_header_list_get_stream (GMimeHeaderList *headers);
+
 void g_mime_header_list_prepend (GMimeHeaderList *headers, const char *name, const char *value);
 void g_mime_header_list_append (GMimeHeaderList *headers, const char *name, const char *value);
 void g_mime_header_list_set (GMimeHeaderList *headers, const char *name, const char *value);
@@ -105,14 +120,12 @@ gboolean g_mime_header_list_remove (GMimeHeaderList *headers, const char *name);
 
 gboolean g_mime_header_list_get_iter (GMimeHeaderList *headers, GMimeHeaderIter *iter);
 
+void g_mime_header_list_foreach (const GMimeHeaderList *headers, GMimeHeaderForeachFunc func, gpointer user_data);
+
 void g_mime_header_list_register_writer (GMimeHeaderList *headers, const char *name, GMimeHeaderWriter writer);
 ssize_t g_mime_header_list_write_to_stream (const GMimeHeaderList *headers, GMimeStream *stream);
 char *g_mime_header_list_to_string (const GMimeHeaderList *headers);
 
-/* for internal use only */
-void g_mime_header_list_set_raw (GMimeHeaderList *headers, const char *raw);
-gboolean g_mime_header_list_has_raw (const GMimeHeaderList *headers);
-
 G_END_DECLS
 
 #endif /* __GMIME_HEADER_H__ */
