@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -27,6 +27,7 @@
 
 #include <gmime/gmime-content-type.h>
 #include <gmime/gmime-disposition.h>
+#include <gmime/gmime-encodings.h>
 #include <gmime/gmime-stream.h>
 #include <gmime/gmime-header.h>
 
@@ -76,6 +77,8 @@ struct _GMimeObjectClass {
 	char *       (* get_headers)   (GMimeObject *object);
 	
 	ssize_t      (* write_to_stream) (GMimeObject *object, GMimeStream *stream);
+	
+	void         (* encode) (GMimeObject *object, GMimeEncodingConstraint constraint);
 };
 
 
@@ -128,6 +131,12 @@ char *g_mime_object_get_headers (GMimeObject *object);
 ssize_t g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream);
 char *g_mime_object_to_string (GMimeObject *object);
 
+void g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
+
+/* Internal API */
+G_GNUC_INTERNAL void g_mime_object_type_registry_init (void);
+G_GNUC_INTERNAL void g_mime_object_type_registry_shutdown (void);
+
 G_END_DECLS
 
 #endif /* __GMIME_OBJECT_H__ */
