@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,14 +59,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 #include <fcntl.h>
 #include <errno.h>
-#ifdef G_OS_WIN32
-#include <io.h>
-#endif
 
 #ifdef HAVE_GETOPT_H
 #define _GNU_SOURCE
@@ -97,7 +92,7 @@ usage (const char *progname)
 static void
 version (const char *progname)
 {
-	printf ("%s - GMime %s\n", progname, VERSION);
+	printf ("%s - GMime %s\n", progname, GMIME_VERSION);
 }
 
 static int
@@ -144,7 +139,7 @@ uuencode (const char *progname, int argc, char **argv)
 	name = argv[optind];
 	
 	/* open our input file... */
-	if ((fd = filename ? open (filename, O_RDONLY) : dup (0)) == -1) {
+	if ((fd = filename ? open (filename, O_RDONLY, 0) : dup (0)) == -1) {
 		fprintf (stderr, "%s: %s: %s\n", progname,
 			 filename ? filename : "stdin",
 			 g_strerror (errno));
